// Output generated from file: D:/programs install/fraclal generators/Fragmentarium-anim/Examples/Historical 3D Fractals/Mandelbulb.frag
// Created:  7.  22:52:18 2015
#info Mandelbulb Distance Estimator
#define providesInit
#include "MathUtils.frag"
#define USE_IQ_CLOUDS
#include "DE-kn2.frag"



#group Mandelbulb

// Number of fractal iterations.
uniform int Iterations;  slider[0,9,100]

// Number of color iterations.
uniform int ColorIterations;  slider[0,9,100]

// Mandelbulb exponent (8 is standard)
uniform float Power; slider[0,8,16]

// Bailout radius
uniform float Bailout; slider[0,5,30]

// Alternate is slightly different, but looks more like a Mandelbrot for Power=2
uniform bool AlternateVersion; checkbox[false]

uniform vec3 RotVector; slider[(0,0,0),(1,1,1),(1,1,1)]

uniform float RotAngle; slider[0.00,0,180]


mat3 rot;
uniform float time;
void init() {
	rot = rotationMatrix3(normalize(RotVector), RotAngle);
	float dummy = time*10.0;
}

// This is my power function, based on the standard spherical coordinates as defined here:
// http://en.wikipedia.org/wiki/Spherical_coordinate_system
//
// It seems to be similar to the one Quilez uses:
// http://www.iquilezles.org/www/articles/mandelbulb/mandelbulb.htm
//
// Notice the north and south poles are different here.
void powN1(inout vec3 z, float r, inout float dr) {
	// extract polar coordinates
	float theta = acos(z.z/r);
	float phi = atan(z.y,z.x);
	dr =  pow( r, Power-1.0)*Power*dr + 1.0;
	
	// scale and rotate the point
	float zr = pow( r,Power);
	theta = theta*Power;
	phi = phi*Power;
	
	// convert back to cartesian coordinates
	z = zr*vec3(sin(theta)*cos(phi), sin(phi)*sin(theta), cos(theta));
}

// This is a power function taken from the implementation by Enforcer:
// http://www.fractalforums.com/mandelbulb-implementation/realtime-renderingoptimisations/
//
// I cannot follow its derivation from spherical coordinates,
// but it does give a nice mandelbrot like object for Power=2
void powN2(inout vec3 z, float zr0, inout float dr) {
	float zo0 = asin( z.z/zr0 );
	float zi0 = atan( z.y,z.x );
	float zr = pow( zr0, Power-1.0 );
	float zo = zo0 * Power;
	float zi = zi0 * Power;
	dr = zr*dr*Power + 1.0;
	zr *= zr0;
	z  = zr*vec3( cos(zo)*cos(zi), cos(zo)*sin(zi), sin(zo) );
}



uniform bool Julia; checkbox[false]
uniform vec3 JuliaC; slider[(-2,-2,-2),(0,0,0),(2,2,2)]

float Mag(vec3 pos) {
vec3 z=pos;
int i=0;
float magold = z.x*z.x + z.y*z.y;
for (i=0; i<16; i++) {
float xn = z.x - 0.1* sin(z.y + 5*sin(2*z.y-0.2*z.x));
float yn = z.y - 0.1* sin(z.x + 5*sin(2*z.x-0.2*z.y));  //yn = ...; // pick whatever you like with sin cos etc add more sines do it yourself 
z.x= xn; z.y= yn;
}
float mag = magold - z.x*z.x - z.y*z.y;
mag = log(1.5+ mag); // smooth
return mag;
}

// Compute the distance from `pos` to the Mandelbox.
float DE(vec3 pos) {
	vec3 z=pos;
	float r;
	float dr=1.0;
	int i=0;
	r=length(z);

	while(r<Bailout && (i<Iterations)) {
		if (AlternateVersion) {
			powN2(z,r,dr);
		} else {
			powN1(z,r,dr);
		}
		z+=(Julia ? JuliaC : pos);
		r=length(z);
		

		z*=rot;
		//z.x=abs(z.y);
		
		if (i<ColorIterations) orbitTrap = min(orbitTrap, abs(vec4(z.x,z.y,z.z,r*r)));
		i++;
	}


	//	if ((type==1) && r<Bailout) return 0.0;
	return 0.5*log(r)*r/dr;
	
	//Use this code for some nice intersections (Power=2)
	//float a =  max(mv1*log(r)*r/dr, abs(pos.y));
	//float b = 1000;
	//if (pos.y>0)  b = 0.5*log(r)*r/dr;
	//return min(min(a, b),
		//	min(0.5*log(r)*r/dr, abs(pos.z)));
	
}



















#preset default
FOV = 0.4
Eye = -7.4089,8.2177,-1.85838
Target = -4.85148,-1.34367,-3.2865
FocalPlane = 1
Aperture = 0
InFocusAWidth = 0
ApertureNbrSides = 5
ApertureRot = 0
ApStarShaped = false
Gamma = 1
ToneMapping = 1
Exposure = 1
Brightness = 1
Contrast = 1
Saturation = 1
GaussianWeight = 1
AntiAliasScale = 2
Bloom = false
BloomIntensity = 0.25
BloomPow = 2
Detail = -3.3
FudgeFactor = 1
MaxDistance = 100
Dither = 0.75
NormalBackStep = 1
AO = 0,0,0,0.76596
AoCorrect = 1
Specular = 0.18367
SpecularExp = 11
CamLight = 1,1,1,1.13846
CamLightMin = 1
Reflection = 0,0,0
SpotGlow = true
SpotLight = 1,1,1,1
LightPos = 0.1124,-7.3034,-1.236
LightSize = 0.12371
LightFallOff = 0
LightGlowRad = 1.0811
LightGlowExp = 1.2
HardShadow = 0.62821
BaseColor = 1,1,1
OrbitStrength = 0
BackgroundColor = 0.207843,0.364706,0.6
GradientBackground = 0
EnableFloor = true
FloorNormal = 0,0,1
FloorHeight = -0.6
FloorColor = 0.290196,0.372549,0.317647
HF_Fallof = 0.01
HF_Const = 0
HF_Intensity = 0
HF_Dir = 0,1,0
HF_Offset = 0
HF_Color = 1,1,1,0.4
HF_Scatter = 0
HF_Anisotropy = 0,0,0
HF_FogIter = 1
HF_CastShadow = false
CloudScale = 1.44442
CloudFlatness = 0
CloudTops = 7.75
CloudBase = 1.3924
CloudDensity = 1
CloudRoughness = 1.52542
CloudContrast = 2.0313
CloudColor = 0.65,0.68,0.7
SunLightColor = 0.7,0.5,0.3
X = 0.5,0.6,0.6,0.7
Y = 1,0.6,0,0.4
Z = 0.8,0.78,1,0.5
R = 0.4,0.7,1,0.12
CycleColors = false
Cycles = 1.1
Up = -0.0224367,0.146942,-0.989653
BloomTaps = 4
DetailAO = -2.08509
MaxRaySteps = 345
Glow = 1,1,1,0.0001
GlowMax = 0
ReflectionsNumber = 0
ShadowSoft = 7.7334
Iterations = 9
ColorIterations = 9
Power = 8
Bailout = 5
AlternateVersion = false
RotVector = 1,1,1
RotAngle = 0
Julia = false
JuliaC = 0,0,0
#endpreset

